/*
 * Tool name   : SecureDelete
 * Description : SecureDelete is a tool that allows you to delete
 *               files permanently and unrecoverable from your Windows
 *               system. There is no remaining copy in the trash can and 
 *               the blocks the file was using are overwritten with random 
 *               data. Once a file was was deleted with SecureDelete the 
 *               data is gone.
 * Version     : 0.1
 * Todo        : -
 * Changes     : - 
 * 
 *
 *
 *
 * License     :
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include <windows.h>
#include <stdio.h>
#include <sys/stat.h>


#define MAX_BUF_SIZE 1024
#define SBOX_SIZE 255



/*
 * Constants, data types and function forward declarations.
 *
 */

unsigned char gSBox[SBOX_SIZE + 1];

void deleteFilesRecursively(char *pBaseDirectory, char *pFilePattern);
void secureDeleteFile(char *pFileName, char *pHomeDirectory);
int encryptBuffer(unsigned char *pKey, unsigned char *pPlainText, int pPlainTextLength, unsigned char *pCipherText, int *pCipherTextLength);
int decryptBuffer(unsigned char *pKey, unsigned char *pCipherText, int pCipherTextLength, unsigned char *pPlainText, int *pPlainTextLength);
int initializeSBox(unsigned char *pKey);




/*
 * Program entry point
 *
 */

int main(int argc, char **argv)
{
  int lRetVal = 0;

  if (argc != 2)
  {
    printf("usage : %s file-pattern\n\n", argv[0]);
  } else {
    deleteFilesRecursively(getenv("HOMEPATH"), argv[1]);
  } // if (argc != 2)
  return(lRetVal);
}





/*
 *
 *
 */

void deleteFilesRecursively(char *pBaseDirectory, char *pFilePattern)
{
  WIN32_FIND_DATA lFileData; 
  HANDLE lSearchHandle;  
  char lFilePattern[MAX_BUF_SIZE + 1]; 
  char lSubDirectory[MAX_BUF_SIZE + 1];
  char lHomeDirectory[MAX_BUF_SIZE + 1];
  char lFileName[MAX_BUF_SIZE + 1];


  if (pBaseDirectory != NULL)
  {

    /*
     * 1. Find all directories.
     *
     */

    ZeroMemory(lHomeDirectory, sizeof(lHomeDirectory));
	strncpy(lHomeDirectory, pBaseDirectory, sizeof(lHomeDirectory));

    if (lHomeDirectory[strlen(lHomeDirectory) - 1] != '\\')
      strcat(lHomeDirectory, "\\");

    ZeroMemory(lFilePattern, sizeof(lFilePattern));
	_snprintf(lFilePattern, sizeof(lFilePattern) - 1, "%s*", lHomeDirectory);

    if ((lSearchHandle = FindFirstFile(lFilePattern, &lFileData)) != INVALID_HANDLE_VALUE) 
    {
      do 
      { 
        /*
         * File is a directory. Do a recursive function call to 
         * check it for other favorite files.
         *
         */

        if(lFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
        {
	      if (strcmp(lFileData.cFileName, ".") && strcmp(lFileData.cFileName, ".."))
	      {
            ZeroMemory(lSubDirectory, sizeof(lSubDirectory));
            _snprintf(lSubDirectory, sizeof(lSubDirectory) - 1, "%s\\%s", pBaseDirectory, lFileData.cFileName);
			deleteFilesRecursively(lSubDirectory, pFilePattern);
          } // if (strcmp(lFileDa...
        } // if(lFileData.dwFileAt...
      } while (FindNextFile(lSearchHandle, &lFileData));

	  FindClose(lSearchHandle);
    } // if ((lSearchHandle = FindFirs




    /*
     * 2. Find all files that match the passed pattern and
     *    and delete them.
     *
     */

    ZeroMemory(lHomeDirectory, sizeof(lHomeDirectory));
	strncpy(lHomeDirectory, pBaseDirectory, sizeof(lHomeDirectory));

    if (lHomeDirectory[strlen(lHomeDirectory) - 1] != '\\')
      strcat(lHomeDirectory, "\\");

    ZeroMemory(lFilePattern, sizeof(lFilePattern));
	_snprintf(lFilePattern, sizeof(lFilePattern) - 1, "%s%s", lHomeDirectory, pFilePattern);

    if ((lSearchHandle = FindFirstFile(lFilePattern, &lFileData)) != INVALID_HANDLE_VALUE) 
    {
      do 
      { 
        /*
         * File for deletion found. Start the deletion process.
         *
         */

        if(! (lFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
        {
          ZeroMemory(lFileName, sizeof(lFileName));
		  _snprintf(lFileName, sizeof(lFileName) - 1, "%s%s", lHomeDirectory, lFileData.cFileName);
          secureDeleteFile(lFileName, lHomeDirectory);
        } // if(lFileData.dwFileAt...
      } while (FindNextFile(lSearchHandle, &lFileData));

	  FindClose(lSearchHandle);
    } // if ((lSearchHandle = FindFir...
  } // if (pBaseDirectory != ...
}




/*
 * Function that deletes a file securely.
 * Overwrite the file content with random data,
 * rename the filename to something else and 
 * finally delete the file.
 *
 */

void secureDeleteFile(char *pFileName, char *pBaseDirectory)
{
  int lFileSize = 0;
  int lCipherTextLength = 0;
  DWORD lBytesRead = 0;
  DWORD lBytesWritten = 0;
  DWORD lTickCount = 0;
  FILETIME lCreationTime;
  FILETIME lLastAccessTime;
  FILETIME lLastWriteTime;
  char *lFileContent = NULL;
  char *lNewFileContent = NULL;
  char lKey[MAX_BUF_SIZE + 1];
  char lNewFileName[MAX_BUF_SIZE + 1];
  char lBaseDirectory[MAX_BUF_SIZE + 1];
  HANDLE lFileHandle = INVALID_HANDLE_VALUE;
  struct _stat lFileStat;
  HANDLE lFileMap = INVALID_HANDLE_VALUE;
  char *lBuff = NULL;


  /*
   * Initialization.
   *
   */

  ZeroMemory(lBaseDirectory, sizeof(lBaseDirectory));
  strncpy(lBaseDirectory, pBaseDirectory, sizeof(lBaseDirectory));

  if (lBaseDirectory[strlen(lBaseDirectory) - 1] != '\\')
    strcat(lBaseDirectory, "\\");



  /*
   * Step 1 : Overwrite the file content with random data. (FILEMAPPING!!)
   *
   */

  if ((lFileHandle = CreateFile(pFileName, GENERIC_READ|GENERIC_WRITE, FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL)) != INVALID_HANDLE_VALUE)
  {
    lFileSize = GetFileSize(lFileHandle, NULL);

    if ((lFileContent = (char *) HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, lFileSize * 2)) != NULL)
    {
      if ((lNewFileContent = (char *) HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, lFileSize * 2)) != NULL)
      {

        /*
         * Create new file name and new file content.
         *
         */

        ReadFile(lFileHandle, lFileContent, lFileSize,  &lBytesRead, NULL);
		ZeroMemory(lKey, sizeof(lKey));

        lTickCount = GetTickCount();
        _snprintf(lKey, sizeof(lKey) - 1, "%d", lTickCount);
        initializeSBox(lKey);
        encryptBuffer(lKey, lFileContent, lFileSize, lNewFileContent, &lCipherTextLength);


        /*
         * Write new file content via file mapping.
         *
         */

        if ((lFileMap = CreateFileMapping(lFileHandle, NULL, PAGE_READWRITE, 0, lFileSize, "memMappedFile")))
        {
          if ((lBuff = MapViewOfFile(lFileMap, FILE_MAP_WRITE, 0, 0, 0)))
          {
            CopyMemory((char *) lBuff, (char *) lNewFileContent, lCipherTextLength);
            UnmapViewOfFile(lBuff);
          } // if ((Buff = MapViewOfFile...

		  CloseHandle(lFileMap);
        } // if ((lFileMap = CreateFileMapp...

        HeapFree(GetProcessHeap(), 0, lNewFileContent);
	  } // if ((lNewFileConte...

      HeapFree(GetProcessHeap(), 0, lFileContent);
	} // if ((lFileContent = (char...
    CloseHandle(lFileHandle);
  } // if ((lFileHandle = Crea...



  /*
   * Step 2 : Rename the file to a random file name.
   *
   */

  ZeroMemory(lNewFileName, sizeof(lNewFileName));

  lTickCount = GetTickCount();
  GetFileTime(lFileHandle, &lCreationTime, &lLastAccessTime, &lLastWriteTime);
  _snprintf(lNewFileName, sizeof(lNewFileName) - 1, "%s%d%d%d%d.bin", lBaseDirectory, lFileSize, lCreationTime.dwHighDateTime, lTickCount, lCreationTime.dwLowDateTime);

  MoveFile(pFileName, lNewFileName);



  /*
   * Step 3 : Delete the file.
   *
   */

  ZeroMemory(&lFileStat, sizeof(lFileStat));
  if (_stat(pFileName, &lFileStat) == 0)
    DeleteFile(pFileName);

  ZeroMemory(&lFileStat, sizeof(lFileStat));
  if (_stat(lNewFileName, &lFileStat) == 0)
    DeleteFile(lNewFileName);

}




/*
 * Initialize the RC4 sbox
 *
 */

int initializeSBox(unsigned char *pKey)
{
  int lLength = 0;
  int lCounter1 = 0;
  int lCounter2 = 0;
  int lCounter3 = 0;

  lLength = strlen((const char *) pKey);

  for (lCounter1 = 0; lCounter1 < 256; lCounter1++)
    gSBox[lCounter1] = lCounter1;

  for (lCounter1 = 0; lCounter1 < 256; lCounter1++)
  {
    lCounter2 = (lCounter2 + gSBox[lCounter1] + pKey[lCounter1 % lLength]) % 256;
    lCounter3 = gSBox[lCounter1];
    gSBox[lCounter1] = gSBox[lCounter2];
    gSBox[lCounter2] = lCounter3;
  }

  return(0);
}



/*
 * Encrypt a data buffer
 *
 */

int encryptBuffer(unsigned char *pKey, unsigned char *pPlainText, int pPlainTextLength, unsigned char *pCipherText, int *pCipherTextLength)
{
  int lCounter1 = 0;
  int lCounter2 = 0;
  int lCounter3 = 0;
  int lCounter4 = 0;
  char *lPlainText = NULL;
  unsigned char lKey[MAX_BUF_SIZE + 1];
  int lRetVal = 0;



  if ((lPlainText = (char *) HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, pPlainTextLength + 1)) == NULL)
  {
    lRetVal = 1;
	goto END;
  }

  ZeroMemory(lKey, sizeof(lKey));

  CopyMemory((char *) lKey, (char *) pKey, sizeof(lKey) - 1);
  CopyMemory((char *) lPlainText, (char *) pPlainText, pPlainTextLength);

  initializeSBox(lKey);
  *pCipherTextLength = 0;


  for (lCounter1 = 0; lCounter1 < pPlainTextLength; lCounter1++)
  {
    lCounter2 = (lCounter2 + 1) % 256;
    lCounter3 = (lCounter3 + gSBox[lCounter2]) % 256;
    lCounter4 = gSBox[lCounter2];
    gSBox[lCounter2] = gSBox[lCounter3];
    gSBox[lCounter3] = lCounter4;

    lCounter4 = (gSBox[lCounter2] + gSBox[lCounter3]) % 256;
    pCipherText[lCounter1] = lPlainText[lCounter1] ^ gSBox[lCounter4];
  }

  *pCipherTextLength = lCounter1;

END:

  if (lPlainText != NULL)
    HeapFree(GetProcessHeap(), 0, lPlainText);

  return(lRetVal);
}



/*
 * Decrypt a data buffer
 *
 */

int decryptBuffer(unsigned char *pKey, unsigned char *pCipherText, int pCipherTextLength, unsigned char *pPlainText, int *pPlainTextLength)
{
  return(encryptBuffer(pKey, pCipherText, pCipherTextLength, pPlainText, pPlainTextLength));
}

