/*
 * Tool name   : ShutDown
 * Description : Example source to shut a system down, reboot it or put
 *               it into standby. It's nice to know how to do it, but
 *               actually ... it's rather unspectacular :)
 * Author      : Ruben Unteregger
 * Homepage    : http://www.megapanzer.com
 *
 * Version     : 0.1
 * OS          : Tested on Microsoft Windows XP
 * Todo        : -
 *
 * Changes     : -
 * 
 * 
 *
 *
 *
 * License     :
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 */

#include <stdio.h>
#include <windows.h>
#include <PowrProf.h>

#pragma comment(lib, "PowrProf.lib")
#pragma comment(lib, "user32.lib")
#pragma comment(lib, "advapi32.lib")


void printUsage(char *pAppName);
int systemShutdown(UINT pParams);





int main(int argc, char* argv[])
{
  int lRetVal = 0;
  UINT lFlags = 0;

  if (argc > 1)
  {
    if (strcmp(argv[1], "-h") == 0)
	{
      systemShutdown(EWX_POWEROFF|EWX_FORCE);
	} else if (strcmp(argv[1], "-l") == 0) {
      ExitWindowsEx(EWX_LOGOFF, EWX_FORCEIFHUNG);
	} else if (strcmp(argv[1], "-r") == 0) {
      systemShutdown(EWX_REBOOT|EWX_FORCE);
	} else if (strcmp(argv[1], "-s") == 0) {
      SetSuspendState(FALSE, TRUE, FALSE);
	} else {
      printUsage(argv[0]);
	}


  } else {
    printUsage(argv[0]);
  }


  return(lRetVal);
}






void printUsage(char *pAppName)
{
  printf("\n\nUsage : %s -h|-l|-r|-s\n\n", pAppName);
  printf("  -h : shut the system down\n");
  printf("  -l : log off user\n");
  printf("  -r : reboot the system\n");
  printf("  -s : suspend the system\n\n");
}




int systemShutdown(UINT pParams)
{
  HANDLE lHToken; 
  TOKEN_PRIVILEGES lTokPriv; 
  int lRetVal = 0;
 
  // Get a token for this process. 
 
  if (!OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES |TOKEN_QUERY, &lHToken))
  {
    lRetVal = 1;
    goto END;
  }


  // Get the LUID for the shutdown privilege. 
  LookupPrivilegeValue(NULL, SE_SHUTDOWN_NAME, &lTokPriv.Privileges[0].Luid); 
 
  lTokPriv.PrivilegeCount = 1;  // one privilege to set    
  lTokPriv.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED; 
 

  // Get the shutdown privilege for this process. 
  AdjustTokenPrivileges(lHToken, FALSE, &lTokPriv, 0, (PTOKEN_PRIVILEGES)NULL, 0); 
 
  if (GetLastError() != ERROR_SUCCESS) 
  {
    lRetVal = 2;
    goto END;
  }



  // Shut down the system and force all applications to close. 
 
  if (!ExitWindowsEx(pParams, SHTDN_REASON_MAJOR_OPERATINGSYSTEM |
                     SHTDN_REASON_MINOR_UPGRADE | SHTDN_REASON_FLAG_PLANNED))
  {
    lRetVal = 3;
    goto END;
  }




END:


  return(lRetVal);
}