/*
 * Tool name   : Skype4LogAnalyzer
 * Description : A tool to extract and list the data from the
 *               Skype 4.* logfile . 
 *
 * Version     : 0.1
 * OS          : Tested on Microsoft Windows XP
 * Todo        : There is other that can be extracted. I'll implement this
 *               in release 0.2
 *
 * Changes     : -
 * 
 *
 */


#include <windows.h>
#include <shlwapi.h>
#include <stdio.h>
#include "sqlite3.h"

#include <time.h>
#include <sys/types.h>
#include <sys/timeb.h>



#define snprintf _snprintf
#define MAX_BUF_SIZE 1024
#define SKYPE_DIR "Skype\\"
#define MAX_ARR_SIZE 128



/*
 * Constants, data types and function forward declarations.
 *
 */

#pragma comment(lib, "shlwapi.lib")

static int callbackSkypeContacts(void *NotUsed, int argc, char **argv, char **azColName);
static int callbackFileTransfers(void *NotUsed, int argc, char **argv, char **azColName);
static int callbackChatMessages(void *NotUsed, int argc, char **argv, char **azColName);
void formatTimeStamp(char *pTimeStamp, char *pTSBuffer, int pTSBufferLength);
void printUsage(char *pProgName);




/*
 * Program entry point
 *
 */


int main(int argc, char* argv[])
{
  int lRetVal = 0;
  char lUsersArr[MAX_ARR_SIZE][256];


  if (argc < 2)
  {
    printUsage(argv[0]);
    lRetVal = 1;
    goto END;
  } else if (strncmp(argv[1], "-cm", 3) != 0 && 
             strncmp(argv[1], "-tr", 3) != 0 &&
             strncmp(argv[1], "-ct", 3) != 0 &&
             strncmp(argv[1], "-all", 4) != 0) {
    printUsage(argv[0]);
    lRetVal = 2;
    goto END;
  }

  findSkypeProfiles(lUsersArr, argv[1]);


END:

  printf("Hit return to stop execution ...\n");
  getc(stdin);


  return(lRetVal);
}




/*
 * Tool usage.
 *
 */

void printUsage(char *pProgName)
{
  printf("Usage  : %s -cm|-ct|-tr|-all\n\n\n\n", pProgName);
  printf("Examples ...\n\n\n");
  printf("%s -cm  : print \"chatmsg\" logs\n", pProgName);
  printf("%s -ct  : print \"contacts\" logs\n", pProgName);
  printf("%s -tr  : print \"transfer\" logs\n", pProgName);
  printf("%s -all : print all logs\n\n", pProgName);
}



/*
 * Locate all Skype user profiles.
 *
 */

int findSkypeProfiles(char** pUserArray, char *pLogType)
{
  int lRetVal = 0;
  char lBaseDir[MAX_BUF_SIZE + 1];
  char lDirPattern[MAX_BUF_SIZE + 1];
  char lFullPath[MAX_BUF_SIZE + 1];
  HANDLE hFind = INVALID_HANDLE_VALUE;
  WIN32_FIND_DATA lFindFileData;
  sqlite3 *lDB;
  char *lErrMsg = 0;


  /*
   * Initialisation stuff.
   *
   */

  ZeroMemory(lBaseDir, sizeof(lBaseDir));
  ZeroMemory(lDirPattern, sizeof(lDirPattern));

  snprintf(lBaseDir, sizeof(lBaseDir) - 1, "%s\\%s\\", getenv("APPDATA"), SKYPE_DIR);
  snprintf(lDirPattern, sizeof(lDirPattern) - 1, "%s\\%s\\*", getenv("APPDATA"), SKYPE_DIR);


  
  /*
   * List all files within the Skype base directory and
   * locate all subdirectories with SQLite DB files inside it.
   *
   */

  if ((hFind = FindFirstFile(lDirPattern, &lFindFileData)) == INVALID_HANDLE_VALUE) 
  {
    lRetVal = 1;
    goto END;
  } else {
	// Skip . and .. directories
	FindNextFile(hFind, &lFindFileData);

    do 
    {
	  if(lFindFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
	  {
        ZeroMemory(lFullPath, sizeof(lFullPath));
        snprintf(lFullPath, sizeof(lFullPath) - 1, "%s%s\\main.db", lBaseDir, lFindFileData.cFileName);


        /*
         * Check if DB file exists and if so, execute all queries.
         *
         */

        if (PathFileExists(lFullPath))
		{
          if(sqlite3_open(lFullPath, &lDB) == SQLITE_OK)
          {

            /*
             * List all Skype contacts.
             *
             */

            if (strncmp(pLogType, "-ct", 3) == 0 || strncmp(pLogType, "-all", 4) == 0)
              if(sqlite3_exec(lDB, "SELECT skypename,fullname,languages,country,province,city,emails,homepage FROM contacts", callbackSkypeContacts, 0, &lErrMsg))
                sqlite3_free(lErrMsg);


            /*
             * List all file transfers.
             *
             */

            if (strncmp(pLogType, "-tr", 3) == 0 || strncmp(pLogType, "-all", 4) == 0)
              if(sqlite3_exec(lDB, "SELECT type,partner_handle,partner_dispname,starttime,finishtime,filename,filesize FROM transfers", callbackFileTransfers, 0, &lErrMsg))
                sqlite3_free(lErrMsg);


            /*
             * List all chat messages.
             *
             */

            if (strncmp(pLogType, "-cm", 3) == 0 || strncmp(pLogType, "-all", 4) == 0)
              if(sqlite3_exec(lDB, "SELECT timestamp,author,from_dispname,identities,dialog_partner,body_xml FROM messages", callbackChatMessages, 0, &lErrMsg))
                sqlite3_free(lErrMsg);


            sqlite3_close(lDB);
          } // if(sqlite3_open("...
		} // if (PathFileExist...
	  } // if(lFindFileData.dwFi...
    } while(FindNextFile(hFind, &lFindFileData) != 0);

    FindClose(hFind);
  }

END:

  return(lRetVal);
}





/*
 * List all Skype contact entries.
 *
 */

static int callbackSkypeContacts(void *NotUsed, int argc, char **argv, char **azColName)
{
  int lCounter = 0;


  for(; lCounter < argc; lCounter += 8)
  {
    if (argv[lCounter])
	{
      printf("Skypename : %s\n", argv[lCounter]?argv[lCounter]:"");
      printf("Full name : %s\n", argv[lCounter+1]?argv[lCounter+1]:"");
      printf("Languages : %s\n", argv[lCounter+2]?argv[lCounter+2]:"");
      printf("Country   : %s\n", argv[lCounter+3]?argv[lCounter+3]:"");
      printf("Province  : %s\n", argv[lCounter+4]?argv[lCounter+4]:"");
      printf("City      : %s\n", argv[lCounter+5]?argv[lCounter+5]:"");
      printf("Emails    : %s\n", argv[lCounter+6]?argv[lCounter+6]:"");
      printf("Homepage  : %s\n\n", argv[lCounter+7]?argv[lCounter+7]:"");
	} // if (argv[lCounter]...
  } // for(; lCoun...


  return(0);
}






/*
 * List all transferred files.
 *
 */

static int callbackFileTransfers(void *NotUsed, int argc, char **argv, char **azColName)
{
  int lCounter = 0;
  char lType[MAX_BUF_SIZE + 1];
  char lStartTimeStamp[MAX_BUF_SIZE + 1];
  char lStopTimeStamp[MAX_BUF_SIZE + 1];




  for(; lCounter < argc; lCounter += 7)
  {

    /*
     * Determine transfer type
     *
     */

    ZeroMemory(lType, sizeof(lType));
    if (argv[lCounter])
	{
      if (strncmp(argv[lCounter], "2", 1) == 0)
        snprintf(lType, sizeof(lType) - 1, "Sending file");
	  else
        snprintf(lType, sizeof(lType) - 1, "Receiving file");
	}


    /*
     * Create a readable start time stamp.
     *
     */
    ZeroMemory(lStartTimeStamp, sizeof(lStartTimeStamp));
    if (argv[lCounter+3])
      formatTimeStamp(argv[lCounter+3], lStartTimeStamp, sizeof(lStartTimeStamp) - 1);


    /*
     * Create a readable stop time stamp.
     *
     */
    ZeroMemory(lStopTimeStamp, sizeof(lStopTimeStamp));
    if (argv[lCounter+4])
      formatTimeStamp(argv[lCounter+4], lStopTimeStamp, sizeof(lStopTimeStamp) - 1);


    printf("%s \"%s\" from %s (%s)\n", lType, argv[lCounter+5]?argv[lCounter+5]:"",
		                               argv[lCounter+1]?argv[lCounter+1]:"",
		                               argv[lCounter+2]?argv[lCounter+2]:"");
    printf("\tStarttime        : %s\n", lStartTimeStamp);
    printf("\tFinishtime       : %s\n", lStopTimeStamp);
    printf("\tFilesize         : %s bytes\n\n", argv[lCounter+6]?argv[lCounter+6]:"");
  } // for(; lCoun...

  return(0);
}




/*
 * List all chat messages.
 *
 */

static int callbackChatMessages(void *NotUsed, int argc, char **argv, char **azColName)
{
  int lCounter = 0;
  char lMsgTimeStamp[MAX_BUF_SIZE + 1];


  for(; lCounter < argc; lCounter += 6)
  {


    /*
     * Create a readable message timestamp.
     *
     */
    ZeroMemory(lMsgTimeStamp, sizeof(lMsgTimeStamp));
     if (argv[lCounter])
      formatTimeStamp(argv[lCounter], lMsgTimeStamp, sizeof(lMsgTimeStamp) - 1);   



    printf("Timesgamp     : %s\n", lMsgTimeStamp);
    printf("Author        : %s (%s)\n", argv[lCounter+1]?argv[lCounter+1]:"",
                                        argv[lCounter+2]?argv[lCounter+2]:"");
    printf("Msg Readers   : %s\n", argv[lCounter+3]?argv[lCounter+3]:"");
    printf("dialog partner: %s\n", argv[lCounter+4]?argv[lCounter+4]:"");
    printf("Message       : %s\n\n", argv[lCounter+5]?argv[lCounter+5]:"NULL");
  } // for(; lCoun...

  return(0);
}




/*
 * Transform unix time stamp to human readable time stamp.
 *
 */

void formatTimeStamp(char *pTimeStamp, char *pTSBuffer, int pTSBufferLength)
{
  time_t lTime = 0;
  struct tm *lTS;

  lTime = atoi(pTimeStamp);
  lTS = localtime(&lTime);
  strftime(pTSBuffer, pTSBufferLength, "%Y.%m.%d %H:%M:%S", lTS);
}