/*
 * Tool name   : WebCamStillImage
 * Description : Tool to take a webcam still image.
 * Author      : carrumba
 * Webpage     : http://www.megapanzer.com
 * Version     : 0.1
 * Todo        : -
 *
 * Changes     : -
 * 
 *
 *
 *
 * License     :
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */




#include <stdio.h>
#include <windows.h>
#include <Vfw.h>
#include <Gdiplus.h>
#include <GdiPlusEnums.h>


#define MAX_BUF_SIZE 1024

#pragma comment(lib, "User32.lib")
#pragma comment(lib, "Vfw32.lib")
#pragma comment(lib, "gdiplus.lib")
#pragma comment(lib, "shell32.lib")


using namespace Gdiplus;




/*
 * Constants, data types and function forward declarations.
 *
 */

LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
int convertBMP2JPG(char *pBMPFile, char *pJPGFile);
int GetCodecClsid(const WCHAR* pFormat, CLSID* pClsid);
int convertBMP2JPG(char *pBMPFile, char *pJPGFile);




/*
 * Program entry point
 *
 */

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int lCmdShow)
{
  int lRetVal = 0;
  HWND lWndVideoHandle;
  CAPSTATUS lCapStat;
  HWND lWND;
  WNDCLASSEX lWCEX;
  char *lBMPFileName = "WebCamSnapshot.bmp";
  char *lJPGFileName = "WebCamSnapshot.jpg";
  char *lTitle = "explorer";
  char *lWindowClass = "explorer";


  /*
   * Initialize values
   *
   */

  lWCEX.cbSize = sizeof(WNDCLASSEX);
  lWCEX.style          = CS_HREDRAW | CS_VREDRAW;
  lWCEX.lpfnWndProc    = WndProc;
  lWCEX.cbClsExtra     = 0;
  lWCEX.cbWndExtra     = 0;
  lWCEX.hInstance      = hInstance;
  lWCEX.hIcon          = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_APPLICATION));
  lWCEX.hCursor        = LoadCursor(NULL, IDC_ARROW);
  lWCEX.hbrBackground  = (HBRUSH)(COLOR_WINDOW+1);
  lWCEX.lpszMenuName   = NULL;
  lWCEX.lpszClassName  = lWindowClass;

  if (!RegisterClassEx(&lWCEX))
  {
    lRetVal = 1;
    goto END;
  }

  if (! (lWND = CreateWindow(lWindowClass, lTitle, WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, CW_USEDEFAULT, 500, 100, NULL, NULL, hInstance, NULL)))
  {
    lRetVal = 2;
    goto END;
  }

  ShowWindow(lWND, SW_HIDE);
  UpdateWindow(lWND);
  ZeroMemory(&lCapStat, sizeof(CAPSTATUS));


  /*
   * Capture one webcam frame.
   */


  Sleep(500);
  lWndVideoHandle = capCreateCaptureWindow("WebCam", WS_CHILD, 0, 0, 320, 240,(HWND) lWND, (int) 1);
  capDriverConnect(lWndVideoHandle, 0);
  Sleep(1000);
  capGetStatus(lWndVideoHandle, &lCapStat, sizeof(CAPSTATUS)); 
  capGrabFrame(lWndVideoHandle);
  capEditCopy(lWndVideoHandle);
  capFileSaveDIB(lWndVideoHandle, lBMPFileName);
  capDriverDisconnect(lWndVideoHandle);
  DestroyWindow(lWndVideoHandle);

  convertBMP2JPG(lBMPFileName, lJPGFileName);

  if (GetFileAttributes(lJPGFileName) != INVALID_FILE_ATTRIBUTES)
    ShellExecute(NULL, "open", lJPGFileName, NULL, NULL, 0);


END:

  /*
   * cleaning up before returning
   */

  DeleteFile(lBMPFileName);

  return(lRetVal);
}



/*
 * Window callback function
 *
 */

LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
   switch(msg)
   {
      case WM_LBUTTONDOWN:
      break;
      case WM_CLOSE:
         DestroyWindow(hwnd);
      break;
      case WM_DESTROY:
         PostQuitMessage(0);
      break;
      default:
         return DefWindowProc(hwnd, msg, wParam, lParam);
   }
   return 0;
}




/*
 * Convert from Bitmap format to JPG.
 *
 */

int convertBMP2JPG(char *pBMPFile, char *pJPGFile)
{
  CLSID lCodecClsid;
  EncoderParameters lEncoderParameters;
  long lQuality = 80;
  GdiplusStartupInput lGdiplusStartupInput;
  ULONG lGdiplusToken;
  WCHAR lWCBMPFile[MAX_BUF_SIZE + 1];
  WCHAR lWCJPGFile[MAX_BUF_SIZE + 1];
  int lRetVal = 0;


  ZeroMemory(lWCBMPFile, sizeof(lWCBMPFile));
  ZeroMemory(lWCBMPFile, sizeof(lWCJPGFile));

  mbstowcs(lWCBMPFile, pBMPFile, sizeof(lWCBMPFile) / sizeof(WCHAR));
  mbstowcs(lWCJPGFile, pJPGFile, sizeof(lWCJPGFile) / sizeof(WCHAR));

  GdiplusStartup(&lGdiplusToken, &lGdiplusStartupInput, NULL);
  {
    Image image(lWCBMPFile);

    // Get the CLSID of the JPEG codec.
    if (GetCodecClsid(L"image/jpeg", &lCodecClsid) >= 0)
    {
      lEncoderParameters.Count = 1;
      lEncoderParameters.Parameter[0].Guid = EncoderQuality;
      lEncoderParameters.Parameter[0].Type = EncoderParameterValueTypeLong;
      lEncoderParameters.Parameter[0].NumberOfValues = 1;
      lEncoderParameters.Parameter[0].Value = &lQuality;

      if (image.Save(lWCJPGFile, &lCodecClsid, &lEncoderParameters) != Ok)
        lRetVal = 1;
    } // if (GetCodecClsid(L"image/jpeg", &lCodecClsid) >= 0)
  }

  GdiplusShutdown(lGdiplusToken);

  return(lRetVal);
}








/*
 * Get an array of ImageCodecInfo objects that contain 
 * information about the available image encoders.
 *
 */

int GetCodecClsid(const WCHAR* pFormat, CLSID* pClsid)
{
  UINT lNumberOfEncoders = 0;
  UINT lArraySize = 0;
  ImageCodecInfo* lImageCodecInfo = NULL;
  int lRetVal = 0;
  int lCounter = 0;

  GetImageEncodersSize(&lNumberOfEncoders, &lArraySize);
  if(lArraySize == 0)
  {
    lRetVal = -1;
    goto END;
  }


  if ((lImageCodecInfo = (ImageCodecInfo*) HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, lArraySize)) == NULL)
  {
    lRetVal = -2;
    goto END;
  }

  GetImageEncoders(lNumberOfEncoders, lArraySize, lImageCodecInfo);

  for(lCounter = 0; lCounter < lNumberOfEncoders; ++lCounter)
  {
    if(wcscmp(lImageCodecInfo[lCounter].MimeType, pFormat) == 0 )
    {
      *pClsid = lImageCodecInfo[lCounter].Clsid;
      lRetVal = lCounter; 
      goto END;
    } // if(wcscmp(lImageC...
  } // for(lCounter = 0; lCounter < lNu


END:
  if (lImageCodecInfo != NULL)
    HeapFree(GetProcessHeap(), 0, lImageCodecInfo);


  return(lRetVal);
}
