/*
 * Tool name   : WhoIs
 * Description : A tool to request Whois registration information
 *               from a Whois server. In this tool the
 *               ripe.net server is the chosen one. Change to whatever
 *               you want.
 * Author      : carrumba
 * Webpage     : http://www.megapanzer.com
 * Version     : 0.1
 * OS          : Tested on Microsoft Windows XP
 * Todo        : -
 *
 * Changes     : - 
 * 
 *
 *
 *
 * License     :
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include <stdio.h>
#include <windows.h>

#define MAX_BUF_SIZE 1024
#define WHOIS_SERVER "whois.ripe.net"
#define WHOIS_PORT 43


#pragma comment(lib, "Ws2_32.lib")


/*
 * Constants, data types and function forward declarations.
 *
 */


int whoIs(char *pQueryObject);


char gWhoisError[24][128] = {"WSAStartup() failed.",
                             "gethostbyname() failed.",
                             "socket() failed.",
                             "connect() failed.",
                             "receiving WHOIS serverbanner failed.",
                             "Sending WHOIS request failed.",

};





/*
 * Program entry point.
 *
 */

int main(int argc, char **argv)
{
  int lRetVal = 0;

  
  if (argc > 1)
  {
    if ((lRetVal = whoIs(argv[1])) > 0)
    {
      printf("An error occured while sending the Whois request (%d) : %s\n\n", lRetVal, gWhoisError[lRetVal - 1]); 
    } else {
      printf("\n\nHit enter to stop execution ...\n");
      getc(stdin);
    } // if ((lRetVa...
  } // if (argc > ...


  return(lRetVal);
}



/*
 * Connect to a Whois server and request information
 * about a specific IP address.
 *
 */

int whoIs(char *pQueryObject)
{
  struct sockaddr_in lSockAddr;
  int lRetVal = 0;
  SOCKET lSocket = 0;
  int lBytesCounter = 0;
  char lTempBuffer[MAX_BUF_SIZE * 2 + 1];
  struct hostent *lHostEntry;
  WORD lVersionRequested;
  WSADATA lWSAData;


  /*
   * Prepare socket connection
   *
   */

  lVersionRequested = MAKEWORD(2, 2);

  if (WSAStartup(lVersionRequested, &lWSAData) != 0)
  {
    lRetVal = 1;
    goto END;
  }


  if ((lHostEntry = gethostbyname(WHOIS_SERVER)) == NULL)
  {
    lRetVal = 2;
    goto END;
  }


  if ((lSocket = (SOCKET) socket(AF_INET, SOCK_STREAM, 0)) < 0)
  {
    lRetVal = 3;
    goto END;
  }

  ZeroMemory(&lSockAddr, sizeof(lSockAddr));
  lSockAddr.sin_addr = *(struct in_addr*) lHostEntry->h_addr;
  lSockAddr.sin_family = AF_INET;
  lSockAddr.sin_port = htons(WHOIS_PORT);


  /*
   * Establish connection and receive WHOIS banner of peer server.
   *
   */

  if (connect(lSocket, (struct sockaddr*) &lSockAddr, sizeof(lSockAddr)) == -1)
  {
    lRetVal = 4;
    goto END;
  }

  ZeroMemory(lTempBuffer, sizeof(lTempBuffer));
  lBytesCounter = 0;
  if ((lBytesCounter = recv(lSocket, lTempBuffer, sizeof(lTempBuffer) - 1, 0)) < 0)
  {
    lRetVal = 5;
    goto END;
  }


  /*
   * Send WHOIS query
   *
   */

  ZeroMemory(lTempBuffer, sizeof(lTempBuffer));
  lBytesCounter = 0;
  _snprintf(lTempBuffer, sizeof(lTempBuffer) - 1, "%s\r\n", pQueryObject);
  if (send(lSocket, lTempBuffer, strlen(lTempBuffer), 0) < 0)
  {
    lRetVal = 6;
    goto END;
  }

  ZeroMemory(lTempBuffer, sizeof(lTempBuffer));
  while((lBytesCounter = recv(lSocket, lTempBuffer, sizeof(lTempBuffer) - 1, 0)) > 0)
  {
    printf("%s", lTempBuffer);
    ZeroMemory(lTempBuffer, sizeof(lTempBuffer));
  } // while((lBytesCounter = rec...

END:

  if (lSocket > 0)
	closesocket(lSocket);

  WSACleanup();

  return(lRetVal);
}